/***********************************
* Program:  deffuncts.p
* Author :  Steven Lichtenberg
* Date   :  08/03/2000
* Purpose:  Routine that defines all functions used in the application.
*           This routine will be run as a SUPER procedure and will
*           then be able to be referenced from anywhere in the application.
* Tables Used: Many
* Modified: 01/29/2001  SWL Moved function hascalls to Super to allow access
*                           from multiple locations
*           01/31/2001  SWL Re-added function isopen to check for invoiced orders
*                           and disallow update of items for those orders
*           03/08/2001  SWL Added functions to check for user type and return
*                           whether they are "on the list"
*           03/16/2001  SWL Moved newship from mkord to allow it to be called from
*                           multiple routines
*           05/11/2001  SWL added new function getcomment to return the last 
*                           vendor comment for display
*           08/13/2001  SWL added getrowid function
*           08/19/2001  SWL newship bug fix to display shipped lines properly 
*           09/06/2001  SWL Added custcontact and custphone for AR
*           11/28/2001  SWL BUG FIX - Shipment number reporting incorrectly
***********************************/
&if opsys = "unix" 
&then &glob esc \\n
&else &glob esc \n
&endif

create widget-pool.

PROCEDURE GetMySuperProcedureHandle :
 DEF OUTPUT PARAMETER op_hMySuperProcedure AS HANDLE NO-UNDO.
 ASSIGN op_hMySuperProcedure = THIS-PROCEDURE:HANDLE.
END PROCEDURE.


SUBSCRIBE TO "GetMySuperProcedureHandle":U ANYWHERE.

function getprice returns dec (input p-item as char,
    input p-cust as char,
    input v-qty as int):

    def var vprice as dec no-undo.
    def var indx   as int no-undo.
    def var v-sic  as char no-undo.

    def buffer b-cust for customer.

    find first customer where customer.cust-number = p-cust no-lock no-error.
    find first item where item.company = "smc" and
        item.item-number =  p-item no-lock no-error.
    if not available item then return 0.00.
    if not available customer then return item.retail-price.
    vprice = item.sale-price.
    find pc-center where pc-center.company = "smc""" and
        pc-center.pc-center = customer.pc-center no-lock no-error.
    if available pc-center and
        pc-center.A-factor > 0 then
        vprice = item.retail-price * pc-center.A-factor.

    if (item.item-number begins "9" AND
        item.inventoried) or
        lookup(item.item-number,"502002,652021,652024,652027") <> 0
    then
        vprice = item.retail-price.
    if item.item-number begins "9" and
        item.inventoried AND
    lookup(customer.pc-center,"012,021,031,042,052,056,059,076,077,078,081,088")
    <> 0 then vprice = item.retail-price * 1.125.
    if lookup(item.item-num,"652021,652024,652027") <> 0 and
        lookup(customer.pc-center,"012,021,031") <> 0 then
    vprice = item.retail-price * 1.10.
                            /* find gsa pricing */
    find first b-cust where b-cust.cust-number =
        customer.bill-to-cust  no-lock no-error.
    v-sic = if available b-cust then b-cust.sic-code else
        customer.sic-code.

                   /* no gsa price - check for special pricing */
    do:                      /* search for customer, item & pc-center match */
        find spec-price where spec-price.company = "smc" and
            spec-price.cust-number = customer.bill-to-cust and
            spec-price.item-number = p-item and
            spec-price.pc-center = customer.pc-center no-lock no-error.
                        /* search for discount percentage record for this center  */
        if not available spec-price then
            find spec-price where spec-price.company = "smc" and
                spec-price.cust-number = customer.bill-to-cust and
                spec-price.item-number = " " and
                spec-price.pc-center = customer.pc-center no-lock no-error.
                          /* search for dollar record for this cust */
        if not available spec-price then
            find spec-price where spec-price.company = "smc" and
            spec-price.cust-number = customer.bill-to-cust and
            spec-price.item-number = p-item and
            spec-price.pc-center = " " no-lock no-error.

                             /* search for percent record for this cust */
        if not available spec-price then
            find spec-price where spec-price.company = "smc" and
            spec-price.cust-number = customer.bill-to-cust and
            spec-price.item-number = " " and
            spec-price.pc-center = " " no-lock no-error.
        if available spec-price and
            spec-price.disc-type = false then
        do:

            find item where item.company = "smc" and
                item.item-number = p-item
            no-lock no-error.
            if (lookup(spec-price.apply-to,"L,B") <> 0 and not item.inventoried)
                or (lookup(spec-price.apply-to,"M,B,N,I") <> 0 and item.inventoried)
            then
            do indx = 1 to 10 :
                if v-qty <= spec-price.qty-disc[indx]
                then
                do:
                       /** if not an industrial ticket and
                       ** apply to is "N" then disc
                       **from mfg list ****/
                    if spec-price.apply-to = "N" then
                        vprice = (item.retail-price *
                        (100 - spec-price.disc-perc[indx])) / 100.
                    else if
                        lookup(spec-price.apply-to,"M,B,L") <> 0 then
                        vprice = (item.sale-price *
                            (100 - spec-price.disc-perc[indx])) / 100.
                end. /* if found qty-break */
            end. /* do indx 1 to 10 */
        end. /* percentage type special price record */
        else if available spec-price and
            spec-price.disc-type = true then
        do indx = 1 to 10 :
            if v-qty <= spec-price.qty-sale[indx]
            then do:
                vprice  = spec-price.sale-price[indx].
                leave.
            end. /* if found qty-break */
        end. /* do indx 1 to 10 */
    end. /* special pricing */
    if v-sic begins "95"  /* search for GSA pricing */
    then
    do:
        find first spec-price where spec-price.company = "smc" and
            spec-price.cust-type = "95" and
            spec-price.item-number = item.item-number and
            spec-price.pc-center = " " no-lock no-error.

        if available spec-price then
        sp-block:
        do indx = 1 to 10:                                  /* gsa-price = true. */
            if v-qty <= spec-price.qty-sale[indx] then
            do:
                vprice = spec-price.sale-price[indx].
                leave sp-block.
            end. /*   if v-qty <= spec-price.qty-sale[indx] then */
        end. /* do indx 1 to 10 */
    end. /*   if v-sic begins "95"  - GSA Pricing */
    return vprice.
end function. /* getprice */

function usrcent returns char (input p-idno as char,input pcent as char):
    if pcent <> "" then return pcent.
    find first user-center where user-center.euid = p-idno
        no-lock no-error.

    if not available user-center then return "no".
    else if user-center.pc-center = "all" then return "052".
    else return user-center.pc-center.
end function.  /* usrcent */

function cust-num returns char (input p-idno as char):
    find first usr_login where usr_login.login = p-idno
       no-lock no-error.
    if not available usr_login then
        return "no user".
    find first customer where customer.cust-number = usr_login.company-id
        no-lock no-error.
    if not available customer then
    do:
       return "no Customer".
    end.
    return customer.cust-number.
end function. /* cust-num */

function getcolor returns char(input v-due as char,
    input v-typ as char,
    input v-call as char,
    input-output v-color as char):
    case v-typ:
        when "bg" then
        do:
           if date(v-due) - today <= 0 then v-color =  "#ff4242".  /* red */
           else if date(v-due) - today = 1 then v-color = "#ffff00". /* yellow */
           else v-color = "#e1e1e1".  /* grey */
        end. /* when "bg" */
        otherwise
        do:
            if date(v-due) - today <= 0 then v-color = "#ffffff". /* white */
            else if date(v-due) - today = 1 then v-color = "#000000".  /* black */
            else v-color = "#000000".   /* black */
       end. /* otherwise */
   end case. /* v-typ */
   return v-color.
end function. /* getcolor */

function getcustinf returns char (input v-cust as char):
    def var v-custinfo as char no-undo.

    find first customer where customer.cust-number = v-cust no-lock no-error.
    if not available customer then return "No Customer Info".
    else assign
        v-custinfo = customer.cust-number + " " +
                     customer.cust-name  + " {&esc}" +
                     customer.cust-addr[1] + " {&esc}"
        v-custinfo =  if customer.cust-addr[2] <> "" then v-custinfo +  customer.cust-addr[2] + " {&esc}"
                      else v-custinfo
        v-custinfo = v-custinfo +  customer.city + ", " + customer.state + " " + customer.zip + " {&esc}" +
                     "Phone (" + customer.area-code  + ")" + customer.phone + " {&esc}"
        v-custinfo = if customer.fax <> "" then  customer.area-code + "~-" + customer.fax + " {&esc}"
                     else v-custinfo
        v-custinfo = v-custinfo + " " + customer.contact + " {&esc}"
        v-custinfo = replace(v-custinfo,","," {&esc}").
        v-custinfo = replace(v-custinfo,"~'","").
   return v-custinfo.
end function. /* custinf */

function vendinf returns char(input v-vend as char):
    def var v-vendinf as char no-undo.
    def var icnt      as int  no-undo.

    find first vendor where vendor.vend-number = v-vend no-lock no-error.
    if not available vendor then return "No Vendor Information Available".
    else assign
        v-vendinf = v-vend
        v-vendinf = v-vendinf + " " + vendor.vend-name +  " {&esc} "
           +  vendor.vend-addr[1]  + " {&esc} "
        v-vendinf = if vendor.vend-addr[2] <> "" then v-vendinf + " " +
            vendor.vend-addr[2] + " {&esc}  " else v-vendinf
        v-vendinf = v-vendinf  + " " + vendor.city + " " + vendor.state + " " + vendor.zip + " {&esc} "   
        v-vendinf   = v-vendinf + " Contact: " + vendor.contact + " {&esc} "
        v-vendinf = v-vendinf + " Phone: ~(" + vendor.area-code + "~) " + vendor.phone + "      {&esc}      " 
        + "Fax: ~(" +
                 vendor.fax-area + "~) " + vendor.fax + " {&esc}   " .
        /*
        do icnt = 1 to 4:
            v-vendinf = v-vendinf + vendor.memo[icnt] + " {&esc}\n".
        end. /* do icnt = 1 to 4 */ 
        */
        v-vendinf = replace(v-vendinf,"'","").
    return v-vendinf.
end function. /* vendinf */

function dispval returns char(input p-hand as handle):
    return p-hand:buffer-value.
end function. /* dispval */

function custnam returns char (input pnum as char):
    find first customer where customer.cust-number = pnum no-lock no-error.
    if not available customer then return "No Customer".
    else return customer.cust-name.
end function. /* custnam */
function storenum returns char (input pnum as char):
    find first customer where customer.cust-number = pnum no-lock no-error.
    if not available customer then return "0".
    else return customer.store-num.
end function. /* storenum */
function vendnam returns char(input pnum as char):
    find first vendor where vendor.vend-number = pnum no-lock no-error.
    if not available vendor then return "No Vendor".
    else return vendor.vend-name + " - " + vendor.vend-number.
end function. /* vendnam */
function getpday  returns int (input pcustnum as char,
    input itm as char):
    find customer where customer.cust-number = pcustnum no-lock no-error.
    find first usrcomp where usrcomp.company-id = customer.bill-to-cust
         no-lock no-error.
    if not available usrcomp then
    do:
       find first usrcomp where usrcomp.company-id = "default" no-lock
           no-error.
       if not available usrcomp then return 0.
    end. /* if not available usrcomp */
    case itm:
        when "1" then return usrcomp.emergtime.
        when "2" then return usrcomp.p1time.
        when "3" then return usrcomp.p2time.
        otherwise return usrcomp.p3time.
    end case. /* case itm */
end function. /* getpday */

function getptyp returns log (input pcustnum as char,
    input itm as char):
    find first customer where customer.cust-number = pcustnum no-lock
        no-error.

    find first usrcomp where usrcomp.company-id = customer.bill-to-cust
        no-lock no-error.
    if not available usrcomp then
    do:
       find first usrcomp where usrcomp.company-id = "default" no-lock
           no-error.
       if not available usrcomp then return false.
    end. /* if not available usrcomp */

   case itm:
        when "1" then return usrcomp.emisday.
        when "2" then return usrcomp.p1isday.
        when "3" then return usrcomp.p2isday.
        otherwise return usrcomp.p3isday.
    end case. /* case itm */
end function. /* getptyp */

function custvend returns char (input v-where as char,
    input v-file as char):

    def var hfile             as handle no-undo.
    def var h-query           as handle no-undo.
    def var h-buffer          as handle no-undo.
    def var h-fld             as handle no-undo.
    def var nmlst             as char no-undo.
    def var i                 as int  no-undo.
    def var isdone            as log no-undo.

    create buffer h-buffer for table v-file.
    create query h-query.
    h-query:set-buffers(h-buffer).
    h-query:query-prepare(v-where).
    h-query:query-open.
    repeat:
        h-query:get-next.
        if h-query:query-off-end then leave.
        case v-file:
            when "customer" then
            do:
                h-fld = h-buffer:buffer-field("cust-number").
                if can-find(first calls where calls.cust-number = h-fld:buffer-value) then
                    nmlst = if nmlst eq "" then
                        h-fld:buffer-value else nmlst + "," + h-fld:buffer-value.
            end.
            when "vendor" then
            do:
                h-fld = h-buffer:buffer-field("vend-number").
                    if can-find(first calls where calls.vend-number = h-fld:buffer-value) then
                        nmlst   = if nmlst eq "" then
                        h-fld:buffer-value else nmlst + "," + h-fld:buffer-value.
            end.
        end case. /* v-file */
    end. /* repeat */
    return nmlst.
    delete object hfile no-error.
    delete object h-query no-error.
    delete object h-buffer no-error.
    delete object h-fld no-error.
end function. /* custvend */

function hascalls returns char (input pcustnum as char):
    def var calllst as char no-undo.
    open query q preselect each calls where
       calls.cust-number = pcustnum and
        calls.ordstat = "open" no-lock.
    if num-results("q") = 0 or num-results("q") = ? then return "".
    else repeat:
        get next q.
        if query-off-end("q") then leave.
        calllst = if calls.po-number <> "" then
            calllst + calls.po-number + "  "
          else calllst + /* replace(calls.calldesc,"~n","")*/ "  ".
    end. /* repeat */
    return calllst.
end function. /* hascalls */

function isopen returns log(input porder as char).
    find order where order.company = "smc" and
        order.order-number = porder no-lock no-error.
    if available order and order.order-status = "c" then return no.
    else return yes.
end function. /* isopen */

function isable returns log(input pitem as char,
    input pcust as char).
    /***** beginning function for security allow of features ******/

    def var isok as log no-undo.
    find dispallow where dispallow.dspitm = pitem no-lock no-error.
    if not available dispallow then return yes. /* default permission is allowed */
    find first euid where euid.user-id = pcust no-lock no-error.
    if available euid then if can-do(dispallow.perms,euid.euid) then
        isok = yes.
    if not isok then
        if can-do(dispallow.perms,pcust) then isok = yes.
    return isok.
end function. /* isable */

function onhand returns decimal (input itmno as char,
    input whse as char):
    find first inv-balance where inv-balance.company = "smc" and
        inv-balance.item-number = itmno and
        inv-balance.warehouse   = whse no-lock no-error.
    if not available inv-balance then return 0.0.
    else return inv-bal.
end. /*  onhand */

function getpd returns int:
    find ar-control no-lock no-error.
    return curr-acct-pr.
end function. /* function getpd */

function getyr returns int:
    find ar-control no-lock no-error.
    return curr-acct-yr.
    return curr-acct-yr.
    return curr-acct-yr.
end function. /* getyr */

function newship returns log(input pordnum as char,itmnum as char,qty as dec,lineno as int):
    find first order-line where order-line.company = "smc" and
        order-line.order-number = pordnum and
        order-line.item-number  = itmnum and
        order-line.line-number  = lineno and
        order-line.shipped-qty  = qty no-lock no-error.
    if available order-line then return no.
    else return yes.
end. /* newship */

function getcomment returns char(input pnum as char):
    define var hqry as handle no-undo.
    define var hbuf as handle no-undo.
    define var hfld as handle no-undo.
    define var vcomment as char no-undo.
    
    create query hqry.
    create buffer hbuf for table "vndcomment".
    hqry:set-buffers(hbuf).
    hqry:query-prepare("for each vndcomment no-lock where vend-number = ~'" +
        pnum + "~' by seqnum").
    message hqry:prepare-string.
    /* if not hqry:query-open then return "". */
    hqry:query-open no-error.
    if error-status:error then message error-status:get-message(1).
    hqry:get-last.
    if not hqry:query-off-end then
    assign
        hfld = hbuf:buffer-field("comment")
        vcomment = hfld:buffer-value.
    delete object hqry no-error.
    delete object hbuf no-error.
    assign
        hqry = ?
        hbuf = ?.
    return vcomment.
end function. /* getcomment */
function getshipno returns char(input porder as char, input pcust as char,input shipnum as int):
    if shipnum <> ? and shipnum <> 0 then return string(shipnum).
    find first shipment where shipment.company = "smc" and
        shipment.cust-number = pcust and
        shipment.order-number = porder no-lock no-error.
    if available shipment then return string(shipment.shipment-num).
    else return "".
end function. /* getshipno */


function avgclose returns decimal (input p-center as char,
    input p-file as char,
    input p-where as char,
    input p-names as char,
    input typ as char).
    
    def var h-buff      as handle no-undo.
    def var h-qry       as handle no-undo.
    def var h-itmno     as handle no-undo.
    def var h-firstbuff as handle no-undo.
    def var h-fqry      as handle no-undo.
    def var h-fld       as handle no-undo. 
    def var hstdat      as handle no-undo.
    def var hendat      as handle no-undo.
    
    def var totday      as int no-undo.
    def var numords     as int no-undo.
    def var vavg        as dec no-undo.
    def var isok        as log.
    
    create buffer h-buff for table p-file.
    create buffer h-firstbuff for table p-file.
    create query h-qry.
    create query h-fqry.
        
    h-qry:set-buffers(h-buff).
    h-fqry:set-buffers(h-firstbuff).
    isok = h-qry:query-prepare("for each " + p-file + " no-lock where " + p-where).
    if not isok then {&out} 'h-qry:query-prepare failed <br>'.
    isok = h-qry:query-open.
    if not isok then 
    do:
        {&out} 'h-qry:query-open failed <br>'.
        return 0.0.
    end. 
    repeat:
        h-qry:get-next.
        if h-qry:query-off-end then leave.
        h-fld = h-buff:buffer-field("callno").   
        if p-names <> "" then
        do:
           find first calls where calls.callno = h-fld:buffer-value no-lock 
              no-error.
           if not available calls then next.
           if typ = "cust" and available calls and 
              lookup(calls.cust-number,p-names) = 0 then
              next.     
           else if typ = "vend" and available calls and lookup(calls.vend-number,p-names) = 0 then 
              next. 
        end. /* if names <> "" */           
        isok = h-fqry:query-prepare("for each " + h-firstbuff:name + " no-lock where 
            callno = ~'" + h-fld:buffer-value + "~' and statcd = ~'open~' and
            stsubcd = ~'new~'").
        if not isok then {&out} 'in avgclose query-prepare failed <br>'.
        
        isok = h-fqry:query-open.
        if not isok then 
        do:
            {&out} 'in avgclose query-open failed <br>'.
            return 0.0.
        end.
        h-fqry:get-next.
        if h-fqry:query-off-end then next.     
       
        assign
            numords = numords + 1
            hstdat  = h-buff:buffer-field("stdate")
            hendat  = h-firstbuff:buffer-field("stdate")
            totday  = totday + ((hstdat:buffer-value - hendat:buffer-value) + 1).
    end.
    vavg = totday / numords.
    if vavg = ? then return 0.0.
    else return vavg.   
end function. /* avgclose */  
function getrowid returns char (input vtable as char, input vwhere as char):
    def var hqry as handle no-undo.
    def var hbuffer as handle no-undo.
    def var vstring as char no-undo.

    create buffer hbuffer for table vtable.
    create query hqry.
    hqry:set-buffers(hbuffer).
    hqry:query-prepare("for each " + vtable + " no-lock " + vwhere).
    hqry:query-open.
    vstring = if not hqry:get-next then ""
    else string(hbuffer:rowid).
    delete object hqry no-error.
    delete object hbuffer no-error.
    return vstring.
end function. /* getrowid */
function usrname returns char(input vid as char):
    find user-name where user-name.u-id = vid no-lock no-error                                                              .
    if available user-name then return user-name.first-name + " " + 
        user-name.last-name.
    else return "".
end. /* usrname */                                      
function custcontact returns char (input pidno as char):
    find customer where customer.cust-number = pidno no-lock no-error.
    if not available customer then return "Customer Not Found".
    if customer.contact <> "" then return customer.contact.
    else return "No Contact Info".
end function. /* custcontact */
function custphone returns char (input pidno as char):
    find customer where customer.cust-number = pidno no-lock no-error.
    if not available customer then return "Customer Not Found".
    else return "(" + customer.area-code + ")" + customer.phone.
end function. /* custphone */
function terms returns char (input pidno as char):
    find customer where customer.cust-number = pidno no-lock no-error.
    if not available customer then return "Customer Not Found".
    find terms where terms.terms-code = customer.terms-code no-lock no-error.
    if not available terms then return "No Terms Found".
    return terms.description.
end function. /* terms */
                            
function getrep returns char (input vid as char,
    input vcust as char,
    input ptype as char):
    case ptype:
        when "cust" then
        do:
            find first report-code no-lock where 
               report-code.company = 'smc' and  
               report-code.report-file = 'customer' and 
               report-code.report-categ = 'aged' and 
               report-code.k-cde = vcust and 
               lookup(report-code.report-code,"pnk,govt,primary,11/97") = 0
            no-error.
            if available report-code then return report-code.report-code.
            else  find first reccd where reccd.user-id = vid no-lock no-error.
            if not available reccd then return "".
            else return reccd.repcd.
        end. /* when "cust" */ 
        otherwise
        do:
            find first reccd where reccd.user-id = vid no-lock no-error.
            if not available reccd then return "".
            else return reccd.repcd.
        end. /* otherwise */
    end case. /* ptype */
end function. /* getrep */



