
/***************************************************************
* Program: getvfxstxml.p.
* Author : Steven Lichtenberg
* Date   : 01/30/2001
* Purpose: Background routine to generate an XML fax status request
*          based on a particular document number and obtain the
*          status back
* Tables Used:  Nome
* Modified: 01/09/2002 SWL Added service reference from XML submit enhancements
****************************************************************/

&glob filenm faxstat    /***** Change to your status table  ****/
&glob updflds fxresult,fxreason
/*** CSL of status result and reason fields in table ***/

{src/web/method/cgidefs.i}
function opensock returns handle(input purl as char,
    input pport as char) in super.
function closesock returns logical(
    input hclose as handle) in super.

def input parameter hbuf as handle no-undo.  /* handle to query buffer */

def var hosturl  as char    no-undo.
def var hostport as char    no-undo.
def var hsock    as handle  no-undo.
def var hfunc    as handle  no-undo.
def var httab    as handle  no-undo.
def var hxdoc    as handle  no-undo.
def var icnt     as int     no-undo.

def var hrec     as handle  no-undo.

def var mdocbuf  as memptr  no-undo.
def var respbuf  as memptr  no-undo.
def var callno   as char    no-undo.

run sockets.p persistent set hfunc.

session:add-super-procedure(hfunc).
assign
    hosturl = "172.16.10.35"  /***** Change to your URL *****/
    hostport = "2996".    /***** VsiFAX uses port 2996 for XML transfers  */


/* open the socket and pass each XML document to the xml server */

hsock = opensock(hosturl,hostport).
/* hsock:set-read-response-procedure("mkstatus",this-procedure).     */


run mkxdoc(input hbuf,
    output mdocbuf,
    output callno).
        /**** this next part seems silly and inefficient here but the VSIfax XML daemon
          issues a server disconnect after each response so we need to reconnect
          for each iteration of the temp table
    ****************************************/
if not hsock:connected() then hsock = opensock(hosturl,hostport).
hsock:write(mdocbuf,1,get-size(mdocbuf)).
                    /****
                        this wait-for will block for input from
                        the socket until there is data coming back
                    *****/
/*
wait-for read-response of hsock.
*/
run mkstatus(input callno).

if error-status:error then return error-status:get-message(1).

procedure mkxdoc:
    def input parameter hrecord as handle no-undo.
    def output parameter docbuf as memptr no-undo.
    def output parameter callno as char   no-undo.

    def var hdoc    as handle no-undo.
    def var hroot   as handle no-undo.
    def var haccnt  as handle no-undo.
    def var hid     as handle no-undo.
    def var hrow    as handle no-undo.
    def var htext   as handle no-undo.
    def var hfield  as handle no-undo.
    def var isok    as log no-undo.

    create x-document hdoc.
    create x-noderef hroot.
    create x-noderef haccnt.
    create x-noderef htext.
    create x-noderef hid.
    create x-noderef hrow.

    hdoc:create-node(hroot,"fax-status","ELEMENT").
    hdoc:append-child(hroot).
    hdoc:create-node(haccnt,"account","ELEMENT").
    hroot:append-child(haccnt).
    hroot:set-attribute("results-format","xml").
    hroot:set-attribute("report-type","full").
    hdoc:create-node(hid,"id","ELEMENT").
    haccnt:append-child(hid).
    hdoc:create-node(htext,"","TEXT").
    hid:append-child(htext).
    htext:node-value="smfax".  /* substitute the name of your fax user
                                  as defined in VSIFAX config */
    hdoc:create-node(hrow,"service-reference","ELEMENT").
                      /****** the Service reference refers to the request ID
                              returned by the VFX server.  This should be captured
                              when the fax is submitted (it is the return value
                              of the submit request) and probably should be in the
                              database somewhere.  Service reference is all that is
                              required to obtain a status response.  In our case we
                              pass a blank for this as it is required and then use the
                              option application reference to find the fax request.
                              We pass the call number as application reference and
                              submit it using the -t arf= parameter to vfx
                       *******/
    hroot:append-child(hrow).
    hdoc:create-node(htext,"","TEXT").
    hrow:append-child(htext).
    hfield = hrecord:buffer-field("vsiid").
    htext:node-value = if hfield:buffer-value <> "" then
        hfield:buffer-value else "".
    hdoc:create-node(hrow,"application-reference","ELEMENT").
    hroot:append-child(hrow).
    hdoc:create-node(htext,"","TEXT").
    hrow:append-child(htext).
    assign

        hfield           = hrecord:buffer-field("callno")
        htext:node-value = hfield:buffer-value
        callno           = hfield:buffer-value.

  /* hdoc:SAVE("file","test.xml").   */

    isok = hdoc:SAVE("memptr",docbuf) no-error.

    delete object hroot.
    delete object haccnt.
    delete object hdoc.
    delete object hid.
    delete object hrow.
    delete object htext.
end procedure.  /* mkxdoc */

procedure getxmlstat:
    def input parameter memstat as memptr no-undo.
    def input parameter callno  as char no-undo.
message "in getxmlstat" view-as alert-box.
    def var xdoc      as handle no-undo.
    def var hroot     as handle no-undo.
    def var hnode     as handle no-undo.
    def var hchild    as handle no-undo.
    def var htext     as handle no-undo.
    def var i         as int    no-undo.
    def var j         as int    no-undo.
    def var k         as int    no-undo.
    def var l         as int    no-undo.
    def var reqresult as char   no-undo.
    def var faxno     as char   no-undo.
    def var idnum     as char   no-undo.
    def var vstate    as char   no-undo.
    def var result    as char   no-undo.
    def var reason    as char   no-undo.
    def var hdata     as handle no-undo.
    def var hno       as handle no-undo.
    def var hfax      as handle no-undo.

    def var isok as log no-undo.

    create x-document xdoc.
    create x-noderef hroot.
    create x-noderef hnode.
    create x-noderef htext.
    create x-noderef hdata.
    create x-noderef hno.
    create x-noderef hfax.
    assign
        xdoc:encoding = "iso8859-1"
        isok = xdoc:load("memptr",memstat,false) no-error.

   message isok  "after load " error-status:get-message(1).
    isok = xdoc:get-document-element(hroot) no-error.
   /* if not isok then return. */
   message isok  " after get hroot " error-status:get-message(1).
    node-block:

    do i = 1 to hroot:num-children:
    message "here".
        isok = hroot:get-child(hnode,i) no-error.
      /*  if not isok then return. */
        message "after get-child".
        case hnode:name:
            when  "request-results"  then
            do:
                reqresult = hnode:get-attribute("status").
                if reqresult <> "normal" then
                do:
                    result = "Request Failed".
                     leave node-block.
                end. /* if reqresult <> "normal" */
            end. /* if hnode:name = "request-results" */
            when "full-status" then
            do j = 1 to hnode:num-children:
                 isok = hnode:get-child(htext,j) no-error.
               /*  if not isok then next. */
                /* skip null values */
                if htext:num-children < 1 then next.
                case htext:name:
                    when "job-status" then
                    do:
                        assign
                            vstate  = htext:get-attribute("current-state")
                            result  = htext:get-attribute("disposition")
                            reason  = if result eq "success" then "Normal" else "".
                    end. /* job-status */
                    when "attempt-status" then
                    do k = 1 to htext:num-children:
                        htext:get-child(hdata,k).
                        if hdata:num-children < 1 then next.
                        case hdata:name:
                            when "recipient" then
                            do l = 1 to hdata:num-children:

                                hdata:get-child(hno,l). /* skip the text for the recipient,
                                                         2 = fax-number element
                                                         3 = fax-number data -type #text */
                                if hno:name = "fax-number" then
                                do:
                                    hno:get-child(hfax,1).
                                    faxno = hfax:node-value.
                                end. /* if hno:name = "fax-number */
                            end. /* when recipient */
                            when "result" then
                            do l = 1 to hdata:num-children:
                                hdata:get-child(hno,l). /* 1 = result text
                                                          2,3 = short-message/text
                                                          4 = long-message
                                                          5 = long message text */
                                if hno:name = "long-message" then
                                do:

                                     hno:get-child(hfax,1).
                                     reason = hfax:node-value.
                                end. /* if hno:name = ""long-message" then */
                            end. /* result-disposition */
                        end case. /* hdata:name */
                    end. /* "attempt-status" */
                end case. /* case htext:name */
            end. /* if hnode:name = "full-status" */
            when "application-reference" then
            do:
                hnode:get-child(htext,1).
                idnum = htext:node-value.
            end. /* when "application-reference" */
        end case. /* hnode:name */
    end. /* do i = 1 to hroot:num-children */
    if idnum = "" then idnum = callno.
    message idnum ' ' result ' ' reason.
    if idnum <> "" then
    run updaterec.p(input "{&filenm}",
            input "{&filenm}.callno = ~'" + idnum + "~' and {&filenm}.fax-number = ~'" +
                faxno + "~' and {&filenm}.fxresult = ~'~'",
            input "{&updflds}",
            input  result + "|" + reason).

    set-size(memstat) = 0.
   /* delete object xdoc.
    delete object hroot.
    delete object hnode.
    delete object htext.
    delete object hdata.
    delete object hno.
    delete object hfax. */
end procedure. /* getxmlstat */

procedure mkstatus:
    def input parameter callno as char no-undo.

    def var isok    as log no-undo.
    def var vlength as int no-undo.
    def var i       as int no-undo.

    set-size(respbuf) = 2056.
    i = 1.
    DO WHILE hsock:GET-BYTES-AVAILABLE() <> 0:
        if i mod 1000 = 0 then
        do:
            message 'in getxml - bytes' hsock:get-bytes-available().
         /*  pause 1. */
        end. /*  if 100 mod i = 0 then  */
        ASSIGN i = i + 1.
        IF i = 10000 THEN LEAVE.
    END.
    vlength = hsock:get-bytes-available().
    if vlength = 0 then return.
/*
    set-size(respbuf) = vlength + 1.

    isok = hsock:read(respbuf,1,vlength ,1).
    */
    isok = hsock:read(respbuf,1,2056,1).
    message get-string(respbuf,1,vlength).
    if isok then run getxmlstat(input respbuf,
        input callno) no-error.
    closesock(hsock).

    if error-status:num-messages > 0 then
       message error-status:get-message(1).
 end procedure. /* mkstatus */



