/***************************************************************
* Program: sendvsixml.p.
* Author : Steven Lichtenberg
* Date   : 01/02/2002
* Purpose: Background routine to generate an XML fax  request
*          based on a particular document number and update the status record to
*          show that it has been sent
* Tables Used:  Nome
* Modified:
****************************************************************/

&glob filenm faxstat    /***** Change to your status table  ****/
&glob updflds fxresult,fxreason,vsiid
/*** CSL of status result and reason fields in table ***/
/*
{src/web/method/cgidefs.i}
*/
function opensock returns handle(input purl as char,
    input pport as char) in super.
function closesock returns logical(
    input hclose as handle) in super.
function usrname returns char(input pid as char)
    in super.
function new-child-node returns handle(input documenthandle as handle,
    input parenthandle as handle,
    input nodename as char,
    input nodevalue as char) forward.
function new-node returns handle(input documenthandle as handle,
    input parenthandle as handle,
    input nodename as char) forward.

def input parameter hostid as char no-undo. /* URL to vsifax server */
def input parameter hostport as char no-undo.  /* generally 2996 but is modifiable */
def input parameter faxno as char no-undo. /* telephone number for fax */
def input parameter faxto as char no-undo. /* who to send to */
def input parameter infile as char no-undo. /* name of file to send */
def input parameter refno as char no-undo. /* reference number back to database */
def input parameter usrid as char no-undo. /* name of the user sending the fax */
def output parameter osuccess as logical no-undo.

def var hsock    as handle  no-undo.
def var hdefs    as handle  no-undo.
def var hfunc    as handle  no-undo.
def var hxdoc    as handle  no-undo.
def var icnt     as int     no-undo.

def var hrec     as handle  no-undo.

def var mdocbuf  as memptr  no-undo.
def var callno   as char    no-undo.
def var reason   as char    no-undo.

publish "GetMySuperProcedureHandle":u(output hdefs).
if not valid-handle(hdefs) then
do:
    run deffuncts.p persistent set hdefs.
    session:add-super-procedure(hdefs).
end. /* if not valid-handle(hdefs) */
PUBLISH "GetSocketHandle":U (OUTPUT hfunc).
IF NOT VALID-HANDLE(hfunc) THEN
do:
    run sockets.p persistent set hfunc.
    session:add-super-procedure(hfunc).
end. /* if not valid-handle(hfunc) */

/* open the socket and pass each XML document to the xml server */

/*
hsock:set-read-response-procedure("mkstatus",this-procedure).
*/

run mkxdoc(input faxno,
    input infile,
    input refno,
    input usrid,
    input faxto,
    output mdocbuf).
        /**** this next part seems silly and inefficient here but the VSIfax XML daemon
          issues a server disconnect after each response so we need to reconnect
          for each iteration of the temp table
    ****************************************/
hsock = opensock(hostid,hostport).
if not hsock:connected() then 
do:
    osuccess = no.
    return "Can not open VSIFAX Socket".
end. /* if not hsock:connected() */    
hsock:write(mdocbuf,1,get-size(mdocbuf)).

/* since we are immediately looking for a response and this routine
will run in batch mode, we do not use a read-response procedure or
a wait-for.  Just run the read routine directly and grab the information
coming back */

run mkstatus(input refno,
    input hsock,
    output reason).
closesock(hsock).
if error-status:error then
do:
    osuccess = no.
    return error-status:get-message(1).
end. /* if error-status:error */
if reason <> "normal" then
do:
    osuccess = no.
    return reason.
end. /* if reason <> "normal" then */
osuccess = yes.
return.
/* build the XML document ******/
procedure mkxdoc:
    def input parameter faxno as char no-undo.
    def input parameter infile as char no-undo.
    def input parameter refno as char no-undo.
    def input parameter usr  as char no-undo.
    def input parameter faxto as char no-undo.
    def output parameter docbuf as memptr no-undo.


    def var encodefil       as memptr no-undo.
    def var filstring       as char no-undo.

    def var hdoc     as handle no-undo.
    def var hroot    as handle no-undo.
    def var haccnt   as handle no-undo.
    def var hnotify  as handle no-undo.
    def var hsender  as handle no-undo.
    def var hrecpt   as handle no-undo.
    def var hsubj    as handle no-undo.
    def var hcontent as handle no-undo.
    def var hbody    as handle no-undo.
    def var hcommand as handle no-undo.
    def var hfax     as handle no-undo.
    def var htext    as handle no-undo.
    def var filcnt   as int    no-undo.
    def var icnt     as int    no-undo.
    def var isok     as log    no-undo.
    def var vcompany as char   init "Safemasters Co., Inc." no-undo.
    create x-document hdoc.
    create x-noderef hroot.
    create x-noderef haccnt.
    create x-noderef hnotify.
    create x-noderef hsender.
    create x-noderef hrecpt.
    create x-noderef hsubj.
    create x-noderef hcontent.
    create x-noderef hcommand.
    create x-noderef htext.
    create x-noderef hfax.
    create x-noderef hbody.
    /* root node */
    hroot = new-node(hdoc,hdoc,"fax-submit").
    /* set attributes of root element "fax-submit" */
    hroot:set-attribute("resolution","fine").
    hroot:set-attribute("coversheet","no"). /* cover pages are created as part of the document 
                                            in our case, change to yes is VSIfax created
                                            cover pages are necessary */
    /**** account node */
    haccnt = new-node(hdoc,hroot,"account").
    /* id = vsifax user ID child of account node */
    /* substitute the name of your fax user
      as defined in VSIFAX config */
    new-child-node(hdoc,haccnt,"id","smfax"). /* change to your vsifax account */
    /* email address of sender */
    new-child-node(hdoc,haccnt,"mail-address",user + "@safemasters.com"). /* change as required */
    /* email notifications set to never to allow for more control
       we will manage notifications of failure so we can give the user
       more information than is in the standard VSIFAX notifications */
    hnotify = new-child-node(hdoc,hroot,"email-notification",user + "@safemasters.com").
    hnotify:set-attribute("when","never"). 
           /* part of the process is to generate a more meaningful email, so
           we do not want VSIFAX to generate the failure notices */
    /**** sender information */
    hsender = new-node(hdoc,hroot,"sender").
    /* personal name (name of the sender */
    new-child-node(hdoc,hsender,"personal-name",usrname(usr)).
    /* company name */
    new-child-node(hdoc,hsender,"company-name",vcompany).
    /* fax number of sender */
    new-child-node(hdoc,hsender,"fax-number","").
    /* Voice number of sender */
    new-child-node(hdoc,hsender,"voice-number","").
    /*** recipient information node */
    hrecpt = new-node(hdoc,hroot,"recipient").
    /* personal name */
    new-child-node(hdoc,hrecpt,"personal-name",faxto).
    /** fax number attaches to recipient */
    hfax = new-node(hdoc,hrecpt,"canonical-fax").
    if length(faxno) > 7 then
    do:
        /* area code */
        new-child-node(hdoc,hfax,"area-code",substr(faxno,1,3)).
        /* phone number */
        new-child-node(hdoc,hfax,"local-number",substr(faxno,4)).
    end. /*     if length(faxno) > 7 then */
    else
    do:
        new-child-node(hdoc,hfax,"local-number",faxno).
    end. /* else */
    /**** subject */
    hsubj = new-child-node(hdoc,hroot,"subject",refno).
    /***** content */
    hcontent = new-node(hdoc,hroot,"content").
    /**** if you are using plain text as the body, uncomment these
    lines ****/
    /*
       hbody = new-child-node(hdoc,hcontent,"body","this is a test of the fax via XML system").
    hbody:set-attribute("filename","inline.txt"). */
    do filcnt = 1 to num-entries(infile):
        if entry(filcnt,infile) = "" then leave.
        /* doencode will base64 encode the document to be faxed and
        will return a memptr.  The memptr will need to be parsed up into
        chunks until 9.1d when you will be able to directly assign a
        memptr to a text attribute of an XML document. */
        run doencode(input entry(filcnt,infile),
            output encodefil).
        hbody = new-node(hdoc,hcontent,"body").

       do icnt = 1 to get-size(encodefil) by 76: 
                  /* each line of the base64 encoded document 
                     is 76 chars long.  Testing shows the encoding 
                     gets garbled if you pass more that that to the Xdoc node 
                   */
            if icnt > 1 then icnt = icnt + 1.
            hdoc:create-node(htext,"","TEXT").
            hbody:append-child(htext).
            htext:node-value = get-string(encodefil,icnt,76).
            if icnt > 1 then icnt = icnt - 1.
        end. /* do icnt = 1 to get-size */

        /*** set transfer encoding to base64 so that pcl documents can be sent
        directly.  If text files will be used as well then modify the
        code to exlude the content-type and transfer-encoding attributes **/
        hbody:set-attribute("filename",entry(1,infile)).
        hbody:set-attribute("content-transfer-encoding","base64").
        hbody:set-attribute("content-type","application/octet-stream").
    end. /* do icnt = 1 to num-entries */

   /**** command-reference  - this is how we will identify the
    information back to the database */
    hcommand = new-child-node(hdoc,hroot,"command-reference",refno).
    hdoc:SAVE("file","test1.xml").

    isok = hdoc:SAVE("memptr",docbuf) no-error.


    delete object hroot.
    delete object haccnt.
    delete object hnotify.
    delete object hsender.
    delete object hrecpt.
    delete object hsubj.
    delete object hcontent.
    delete object hcommand.
    delete object htext.
    delete object hfax.
    delete object hbody.
    delete object hdoc.
end procedure.  /* mkxdoc */
procedure doencode:
    def input parameter infile as char no-undo.
    def output parameter encodefil as memptr no-undo.

    def var encodedirectory as char no-undo.
    def var localfile       as char no-undo.
    assign
        encodedirectory = session:temp-directory
        localfile = encodedirectory + "/" + infile + ".enc".
    run base64encode.p(input infile,
        input localfile).

    file-info:file-name = localfile. 
  /*  file-info:file-name = infile.    */
    input from value(file-info:file-name) binary no-map no-convert.
    set-size(encodefil) = file-info:file-size + 1.
    import encodefil.
    input close.
  /*  os-delete value(file-info:file-name). */
    return.
end procedure. /* doencode */

procedure getxmlstat:
    def input parameter memstat as memptr no-undo.
    def input parameter refno  as char no-undo.
    def output parameter reason    as char   no-undo.

    def var xdoc      as handle no-undo.
    def var hroot     as handle no-undo.
    def var hnode     as handle no-undo.
    def var hchild    as handle no-undo.
    def var htext     as handle no-undo.
    def var i         as int    no-undo.
    def var j         as int    no-undo.
    def var invno   as char   no-undo.
    def var reqresult as char no-undo.
    def var result    as char   no-undo.


    def var isok as log no-undo.

    create x-document xdoc.
    create x-noderef hroot.
    create x-noderef hnode.
    create x-noderef htext.

    assign
        xdoc:encoding = "iso8859-1"
        isok = xdoc:load("memptr",memstat,false) no-error.

        isok = xdoc:get-document-element(hroot) no-error.
        node-block:
        do i = 1 to hroot:num-children:
            isok = hroot:get-child(hnode,i) no-error.
          /*  if not isok then return. */
            case hnode:name:
                when  "request-results"  then
                do:
                    reqresult = hnode:get-attribute("status").
                    if reqresult <> "normal" then
                    do:
                       hnode:get-child(htext,1) no-error.
                       assign
                          result = "Request Failed - "
                          reason = htext:node-value.
                       return result + reason.
                    end. /* if reqresult <> "normal" */
                    else assign
                       result = "Fax Accepted for Delivery"
                       reason = "Normal".
                end. /* if hnode:name = "request-results" */
                when "service-reference" then
                do:
                     isok = hnode:get-child(htext,1) no-error.
                   /*  if not isok then next. */
                    /* skip null values */
                    refno = htext:node-value.
                end. /* if hnode:name = "full-status" */
                when "command-reference" then
                do:
                    hnode:get-child(htext,1).
                    invno = htext:node-value.
                end. /* when "application-reference" */
            end case. /* hnode:name */
        end. /* do i = 1 to hroot:num-children */
        if invno <> "" then
        run updaterec.p(input "{&filenm}",
                input "{&filenm}.callno = ~'" + invno +
                    "~' and {&filenm}.fxresult = ~'~'",
                input "{&updflds}",
                input  result + "|" + reason + "|" + refno).
        /*
xdoc:save("file","resp.xml").
*/
end procedure. /* getxmlstat */

procedure mkstatus:
    def input parameter refno as char no-undo.
    def input parameter hsock as handle no-undo.
    def output parameter reason as char no-undo.
    
    def var isok    as log no-undo.
    def var vlength as int no-undo.
    def var i       as int no-undo.
    def var respbuf  as memptr  no-undo.
    
    set-size(respbuf) = 2056.
    i = 1.
    DO WHILE hsock:GET-BYTES-AVAILABLE() <> 0:
        if i mod 1000 = 0 then
        do:
         /*  pause 1. */
        end. /*  if 100 mod i = 0 then  */
        ASSIGN i = i + 1.
        IF i = 10000 THEN LEAVE.
    END.
    vlength = hsock:get-bytes-available().
    isok = hsock:read(respbuf,1,2056,1).
    if isok then run getxmlstat(input respbuf,
        input refno,
        output reason)
        no-error.
    if error-status:error then return 
        error-status:get-message(1).
    closesock(hsock).
  
 end procedure. /* mkstatus */
 /****************************** Functions **************/
 /* these functions came directly from Geoff Crawfords 
     XML toolkit and appeared in Progressions Magazine.
     For information contact Geoff at
     geoff@innov8cs.com */
function new-child-node returns handle(input documenthandle as handle,
    input parenthandle as handle,
    input nodename as char,
    input nodevalue as char):

    def var newnodehandle as handle no-undo.
    def var newtexthandle as handle no-undo.

    create x-noderef newnodehandle.
    create x-noderef newtexthandle.

    documenthandle:create-node(newnodehandle,nodename,"ELEMENT").
    parenthandle:append-child(newnodehandle).

    documenthandle:create-node(newtexthandle,"","TEXT").
    newtexthandle:node-value = nodevalue.
    newnodehandle:append-child(newtexthandle).

    return newnodehandle.
    delete object newnodehandle.
    delete object newtexthandle.
end function. /* new-child-node */
function new-node returns handle(input documenthandle as handle,
    input parenthandle as handle,
    input nodename as char):

    def var newnodehandle as handle no-undo.
    create x-noderef newnodehandle.

    documenthandle:create-node(newnodehandle,nodename,"ELEMENT").
    parenthandle:append-child(newnodehandle).
    return newnodehandle.
    delete object newnodehandle.
end function. /* new-node */
